#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#The MIT License (MIT)
#
#Copyright (c) <2013-2014> <Colin Duquesnoy and others, see AUTHORS.txt>
#
#Permission is hereby granted, free of charge, to any person obtaining a copy
#of this software and associated documentation files (the "Software"), to deal
#in the Software without restriction, including without limitation the rights
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#copies of the Software, and to permit persons to whom the Software is
#furnished to do so, subject to the following conditions:
#
#The above copyright notice and this permission notice shall be included in
#all copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#THE SOFTWARE.
#
"""
This setup script packages the widgets package of pyQode: pyqode.widgets
"""
from pyqode.widgets.errors_table import QErrorsTable
from pyqode.widgets.home import QHomeWidget
from pyqode.widgets.home import DarkColorScheme
from pyqode.widgets.home import ColorScheme
from pyqode.widgets.interactive import QInteractiveConsole
from pyqode.widgets.property_grid import QPropertyGrid
from pyqode.widgets.property_grid import QColorButton
from pyqode.widgets.property_grid import QTextStyleWidget
from pyqode.widgets.tabs import QCodeEditTabWidget


__version__ = "1.2.2"


__all__ = ["ColorScheme", "DarkColorScheme", "QCodeEditTabWidget",
           "QColorButton", "QErrorsTable", "QHomeWidget", "QInteractiveConsole",
           "QPropertyGrid", "QTextStyleWidget"]
