# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'home.ui'
#
# Created: Wed Nov 13 10:27:38 2013
#      by: pyside-uic 0.2.14 running on PySide 1.1.2
#
# WARNING! All changes made in this file will be lost!

from pyqode.qt import QtCore, QtGui

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(719, 500)
        self.gridLayout = QtGui.QGridLayout(Form)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setObjectName("gridLayout")
        self.frame = QtGui.QFrame(Form)
        self.frame.setMinimumSize(QtCore.QSize(0, 0))
        self.frame.setFrameShape(QtGui.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtGui.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.verticalLayout = QtGui.QVBoxLayout(self.frame)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayoutHeader = QtGui.QHBoxLayout()
        self.horizontalLayoutHeader.setSpacing(0)
        self.horizontalLayoutHeader.setContentsMargins(-1, 0, -1, -1)
        self.horizontalLayoutHeader.setObjectName("horizontalLayoutHeader")
        self.lblIcon = QtGui.QLabel(self.frame)
        self.lblIcon.setText("")
        self.lblIcon.setObjectName("lblIcon")
        self.horizontalLayoutHeader.addWidget(self.lblIcon)
        self.lblTitle = QtGui.QLabel(self.frame)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lblTitle.sizePolicy().hasHeightForWidth())
        self.lblTitle.setSizePolicy(sizePolicy)
        self.lblTitle.setStyleSheet("font: 20pt \"Verdana\";")
        self.lblTitle.setObjectName("lblTitle")
        self.horizontalLayoutHeader.addWidget(self.lblTitle)
        self.verticalLayout.addLayout(self.horizontalLayoutHeader)
        self.horizontalLayoutMain = QtGui.QHBoxLayout()
        self.horizontalLayoutMain.setSpacing(0)
        self.horizontalLayoutMain.setContentsMargins(9, 9, 9, 9)
        self.horizontalLayoutMain.setObjectName("horizontalLayoutMain")
        self.frameRecents = QtGui.QFrame(self.frame)
        self.frameRecents.setFrameShape(QtGui.QFrame.StyledPanel)
        self.frameRecents.setFrameShadow(QtGui.QFrame.Raised)
        self.frameRecents.setObjectName("frameRecents")
        self.verticalLayout_4 = QtGui.QVBoxLayout(self.frameRecents)
        self.verticalLayout_4.setSpacing(0)
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.verticalLayout_2 = QtGui.QVBoxLayout()
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.lblRecents = QtGui.QLabel(self.frameRecents)
        self.lblRecents.setFrameShadow(QtGui.QFrame.Sunken)
        self.lblRecents.setAlignment(QtCore.Qt.AlignCenter)
        self.lblRecents.setObjectName("lblRecents")
        self.verticalLayout_2.addWidget(self.lblRecents)
        self.lwRecents = HomeListWidget(self.frameRecents)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lwRecents.sizePolicy().hasHeightForWidth())
        self.lwRecents.setSizePolicy(sizePolicy)
        self.lwRecents.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.lwRecents.setProperty("cursor", QtCore.Qt.ArrowCursor)
        self.lwRecents.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.lwRecents.setProperty("showDropIndicator", False)
        self.lwRecents.setObjectName("lwRecents")
        self.verticalLayout_2.addWidget(self.lwRecents)
        self.verticalLayout_4.addLayout(self.verticalLayout_2)
        self.horizontalLayoutMain.addWidget(self.frameRecents)
        spacerItem = QtGui.QSpacerItem(20, 20, QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Minimum)
        self.horizontalLayoutMain.addItem(spacerItem)
        self.framequickStart = QtGui.QFrame(self.frame)
        self.framequickStart.setFrameShape(QtGui.QFrame.StyledPanel)
        self.framequickStart.setFrameShadow(QtGui.QFrame.Raised)
        self.framequickStart.setObjectName("framequickStart")
        self.verticalLayout_5 = QtGui.QVBoxLayout(self.framequickStart)
        self.verticalLayout_5.setSpacing(0)
        self.verticalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.verticalLayoutQuickStart = QtGui.QVBoxLayout()
        self.verticalLayoutQuickStart.setSpacing(0)
        self.verticalLayoutQuickStart.setObjectName("verticalLayoutQuickStart")
        self.lblQuickStart = QtGui.QLabel(self.framequickStart)
        self.lblQuickStart.setFrameShadow(QtGui.QFrame.Sunken)
        self.lblQuickStart.setAlignment(QtCore.Qt.AlignCenter)
        self.lblQuickStart.setObjectName("lblQuickStart")
        self.verticalLayoutQuickStart.addWidget(self.lblQuickStart)
        self.lwQuickStart = HomeListWidget(self.framequickStart)
        self.lwQuickStart.setProperty("cursor", QtCore.Qt.ArrowCursor)
        self.lwQuickStart.setObjectName("lwQuickStart")
        self.verticalLayoutQuickStart.addWidget(self.lwQuickStart)
        self.verticalLayout_5.addLayout(self.verticalLayoutQuickStart)
        self.horizontalLayoutMain.addWidget(self.framequickStart)
        self.verticalLayout.addLayout(self.horizontalLayoutMain)
        self.gridLayout.addWidget(self.frame, 0, 0, 1, 1)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        Form.setWindowTitle(QtGui.QApplication.translate("Form", "Form", None, QtGui.QApplication.UnicodeUTF8))
        self.lblTitle.setText(QtGui.QApplication.translate("Form", "<html><head/><body><p><span style=\" font-size:20pt;\">Welcome to %s</span></p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.lblRecents.setText(QtGui.QApplication.translate("Form", "Recents", None, QtGui.QApplication.UnicodeUTF8))
        self.lblQuickStart.setText(QtGui.QApplication.translate("Form", "Quick start Actions", None, QtGui.QApplication.UnicodeUTF8))

from pyqode.widgets._promoted import HomeListWidget